@@ -13,18 +13,18 @@
 %                        Retrieve An Image Via URL.                           %
 %                                                                             %
 %                              Software Design                                %
-%                                John Cristy                                  %
+%                                   Cristy                                    %
 %                              Bill Radcliffe                                 %
 %                                March 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999 ImageMagick Studio LLC, a non-profit organization           %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/license/                                         %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -36,7 +36,8 @@
 %
 %
 */
-
+
+
 /*
   Include declarations.
 */
@@ -51,24 +52,27 @@
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
+#include "magick/module.h"
+#include "magick/pixel-accessor.h"
 #include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/resource_.h"
 #include "magick/string_.h"
-#include "magick/module.h"
+#include "magick/utility.h"
 #if defined(MAGICKCORE_XML_DELEGATE)
-#  if defined(__WINDOWS__)
-#    if defined(__MINGW32__)
-#      define _MSC_VER
-#    else
-#      include <win32config.h>
-#    endif
-#  endif
 #  include <libxml/parser.h>
 #  include <libxml/xmlmemory.h>
+#if defined(LIBXML_FTP_ENABLED)
 #  include <libxml/nanoftp.h>
+#endif
+#if defined(LIBXML_HTTP_ENABLED)
 #  include <libxml/nanohttp.h>
 #endif
+#endif
+#if defined(MAGICKCORE_WINDOWS_SUPPORT)
+#  include <urlmon.h>
+#  pragma comment(lib, "urlmon.lib")
+#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -101,21 +105,24 @@
 extern "C" {
 #endif
 
-static void GetFTPData(void *userdata,const char *data,int length)
+#if defined(MAGICKCORE_XML_DELEGATE) && defined(LIBXML_FTP_ENABLED)
+static void GetFTPData(void *userdata,const char *data,int size)
 {
   FILE
     *file;
 
-  int
-    status;
+  size_t
+    length;
 
   file=(FILE *) userdata;
   if (file == (FILE *) NULL)
     return;
-  if (length <= 0)
+  if (size <= 0)
     return;
-  status=fwrite(data,length,1,file);
+  length=fwrite(data,1,size,file);
+  (void) length;
 }
+#endif
 
 #if defined(__cplusplus) || defined(c_plusplus)
 }
@@ -132,7 +139,9 @@ static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *file;
 
   Image
-    *image;
+    *image,
+    *images,
+    *next;
 
   ImageInfo
     *read_info;
@@ -140,34 +149,76 @@ static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
   int
     unique_file;
 
-  image=(Image *) NULL;
+  images=(Image *) NULL;
+  image=AcquireImage(image_info);
   read_info=CloneImageInfo(image_info);
   SetImageInfoBlob(read_info,(void *) NULL,0);
+#if !defined(MAGICKCORE_WINDOWS_SUPPORT)
+  if (LocaleCompare(read_info->magick,"https") == 0)
+    {
+      MagickBooleanType
+        status;
+
+      /*
+        Leverage delegate to read HTTPS link.
+      */
+      status=InvokeDelegate(read_info,image,"https:decode",(char *) NULL,
+        exception);
+      if (status != MagickFalse)
+        {
+          (void) FormatLocaleString(read_info->filename,MagickPathExtent,
+            "%s.dat",read_info->unique);
+          *read_info->magick='\0';
+          images=ReadImage(read_info,exception);
+          (void) RelinquishUniqueFileResource(read_info->filename);
+          if (images != (Image *) NULL)
+            for (next=images; next != (Image *) NULL; next=next->next)
+              (void) CopyMagickString(next->filename,image->filename,
+                MaxTextExtent);
+        }
+      read_info=DestroyImageInfo(read_info);
+      image=DestroyImage(image);
+      return(images);
+    }
+#endif
+  if (LocaleCompare(read_info->magick,"file") == 0)
+    {
+      (void) CopyMagickString(read_info->filename,image_info->filename+2,
+        MaxTextExtent);
+      *read_info->magick='\0';
+      images=ReadImage(read_info,exception);
+      read_info=DestroyImageInfo(read_info);
+      image=DestroyImage(image);
+      return(GetFirstImageInList(images));
+    }
   file=(FILE *) NULL;
   unique_file=AcquireUniqueFileResource(read_info->filename);
   if (unique_file != -1)
     file=fdopen(unique_file,"wb");
   if ((unique_file == -1) || (file == (FILE *) NULL))
     {
-      read_info=DestroyImageInfo(read_info);
-      (void) CopyMagickString(image->filename,read_info->filename,
-        MaxTextExtent);
       ThrowFileException(exception,FileOpenError,"UnableToCreateTemporaryFile",
-        image->filename);
-      image=DestroyImageList(image);
+        read_info->filename);
+      read_info=DestroyImageInfo(read_info);
+      image=DestroyImage(image);
       return((Image *) NULL);
     }
   (void) CopyMagickString(filename,image_info->magick,MaxTextExtent);
   (void) ConcatenateMagickString(filename,":",MaxTextExtent);
   LocaleLower(filename);
   (void) ConcatenateMagickString(filename,image_info->filename,MaxTextExtent);
-  if (LocaleCompare(read_info->magick,"file") == 0)
+#if defined(MAGICKCORE_WINDOWS_SUPPORT)
+  (void) fclose(file);
+  if (URLDownloadToFile(NULL,filename,read_info->filename,0,NULL) != S_OK)
     {
+      ThrowFileException(exception,FileOpenError,"UnableToOpenFile",
+        filename);
       (void) RelinquishUniqueFileResource(read_info->filename);
-      unique_file=(-1);
-      (void) CopyMagickString(read_info->filename,image_info->filename+2,
-        MaxTextExtent);
+      read_info=DestroyImageInfo(read_info);
+      image=DestroyImage(image);
+      return((Image *) NULL);
     }
+#else
 #if defined(MAGICKCORE_XML_DELEGATE) && defined(LIBXML_FTP_ENABLED)
   if (LocaleCompare(read_info->magick,"ftp") == 0)
     {
@@ -203,11 +254,12 @@ static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
         (const char *) NULL,&type,(const char *) NULL,0);
       if (context != (void *) NULL)
         {
-          int
-            status;
+          ssize_t
+            count;
 
           while ((bytes=xmlNanoHTTPRead(context,buffer,MaxBufferExtent)) > 0)
-            status=fwrite(buffer,bytes,1,file);
+            count=(ssize_t) fwrite(buffer,1,bytes,file);
+          (void) count;
           xmlNanoHTTPClose(context);
           xmlFree(type);
           xmlNanoHTTPCleanup();
@@ -215,15 +267,24 @@ static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
 #endif
   (void) fclose(file);
+#endif
   *read_info->magick='\0';
-  image=ReadImage(read_info,exception);
-  if (unique_file != -1)
-    (void) RelinquishUniqueFileResource(read_info->filename);
+  images=ReadImage(read_info,exception);
+  (void) RelinquishUniqueFileResource(read_info->filename);
+  if (images != (Image *) NULL)
+    for (next=images; next != (Image *) NULL; next=next->next)
+      (void) CopyMagickString(next->filename,image->filename,MaxTextExtent);
   read_info=DestroyImageInfo(read_info);
-  if (image == (Image *) NULL)
-    (void) ThrowMagickException(exception,GetMagickModule(),CoderError,
-      "NoDataReturned","`%s'",filename);
-  return(GetFirstImageInList(image));
+  if (images != (Image *) NULL)
+    GetPathComponent(image_info->filename,TailPath,images->filename);
+  else
+    {
+      (void) ThrowMagickException(exception,GetMagickModule(),CoderError,
+        "NoDataReturned","`%s'",filename);
+      return((Image *) NULL);
+    }
+  image=DestroyImage(image);
+  return(GetFirstImageInList(images));
 }
 
 /*
@@ -246,39 +307,48 @@ static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterURLImage method is:
 %
-%      unsigned long RegisterURLImage(void)
+%      size_t RegisterURLImage(void)
 %
 */
-ModuleExport unsigned long RegisterURLImage(void)
+ModuleExport size_t RegisterURLImage(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("HTTP");
-#if defined(MAGICKCORE_XML_DELEGATE) && defined(LIBXML_HTTP_ENABLED)
+#if defined(MAGICKCORE_WINDOWS_SUPPORT) || \
+    (defined(MAGICKCORE_XML_DELEGATE) && defined(LIBXML_HTTP_ENABLED))
   entry->decoder=(DecodeImageHandler *) ReadURLImage;
 #endif
   entry->description=ConstantString("Uniform Resource Locator (http://)");
-  entry->module=ConstantString("URL");
-  entry->stealth=MagickTrue;
+  entry->magick_module=ConstantString("URL");
+  entry->format_type=ImplicitFormatType;
+  (void) RegisterMagickInfo(entry);
+  entry=SetMagickInfo("HTTPS");
+  entry->decoder=(DecodeImageHandler *) ReadURLImage;
+  entry->description=ConstantString("Uniform Resource Locator (https://)");
+  entry->magick_module=ConstantString("URL");
+  entry->format_type=ImplicitFormatType;
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("FTP");
-#if defined(MAGICKCORE_XML_DELEGATE) && defined(LIBXML_FTP_ENABLED)
+#if defined(MAGICKCORE_WINDOWS_SUPPORT) || \
+    (defined(MAGICKCORE_XML_DELEGATE) && defined(LIBXML_FTP_ENABLED))
   entry->decoder=(DecodeImageHandler *) ReadURLImage;
 #endif
   entry->description=ConstantString("Uniform Resource Locator (ftp://)");
-  entry->module=ConstantString("URL");
-  entry->stealth=MagickTrue;
+  entry->magick_module=ConstantString("URL");
+  entry->format_type=ImplicitFormatType;
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("FILE");
   entry->decoder=(DecodeImageHandler *) ReadURLImage;
   entry->description=ConstantString("Uniform Resource Locator (file://)");
-  entry->module=ConstantString("URL");
-  entry->stealth=MagickTrue;
+  entry->magick_module=ConstantString("URL");
+  entry->format_type=ImplicitFormatType;
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
-
+
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -301,6 +371,7 @@ ModuleExport unsigned long RegisterURLImage(void)
 ModuleExport void UnregisterURLImage(void)
 {
   (void) UnregisterMagickInfo("HTTP");
+  (void) UnregisterMagickInfo("HTTPS");
   (void) UnregisterMagickInfo("FTP");
   (void) UnregisterMagickInfo("FILE");
 }
